/******************************************************************************


	testapp.c

Abstract:

	pcmcia test application: open, ioctl, close the pcmcia device.
 
   In order to run this application on windows 95/98/me SpiroCard, OxiCard or SpirOxCard
   should be inserted first and the card drivers should be installed properly. Then go to
   the Start menu, and select run.  Browse for Win32TestApp.exe.  Select Win32TestApp.exe 
   and type a space and then the mouthpiece number without a dash.
   For example, the run string may look like \Win32TestApp.exe 84400550
   Click OK on the run window to run the application.

--****************************************************************************/

#include <windows.h>
#ifdef DEBUG
#endif
#include <tchar.h>
#include <stdlib.h>
#include <stdio.h>
#include "spirocrd.h"
#include "string.h"
#include <memory.h>
#include <malloc.h>

/* 

FUNCTION: CardInserted
	In Win95/98/me, You must check the registry for the presence of the Spirocard.
	CreateFile will return TRUE, even if card is not inserted.
*/
BOOLEAN CardInserted()
{
		char * SubKey = _T("Config Manager\\Enum");
		HKEY OpenedKey;
		DWORD NumberOfKeys = 0;
		DWORD StringSize = 256;
		HKEY FinalOpenedKey;
		char * str;
//		char compString[256];
		char * FinalKeyString = _T("HardwareKey");
		char KeyNameString[256]; 
		DWORD dw = 500;
		TCHAR * lpsz;

		if (RegOpenKeyEx(HKEY_DYN_DATA,SubKey,0,KEY_ENUMERATE_SUB_KEYS,&OpenedKey) != ERROR_SUCCESS)
		{
			return FALSE;
		}
		// Find the index of the last item, since SpiroCard entry will appear
		// toward the end of the list.
		if (RegQueryInfoKey(OpenedKey,NULL,NULL,NULL,&NumberOfKeys,NULL,NULL,NULL,NULL,NULL,NULL,NULL) != ERROR_SUCCESS)
		{
			RegCloseKey(OpenedKey);
			return FALSE;
		} 
		while((NumberOfKeys > 0))
		{
		 	if (RegEnumKeyEx(OpenedKey,NumberOfKeys -1,KeyNameString, &StringSize, 0,0,0,0) != ERROR_SUCCESS)
			{
				RegCloseKey(OpenedKey);
				return FALSE;
			} 
			if (RegOpenKeyEx(OpenedKey,KeyNameString,0,KEY_READ,&FinalOpenedKey) != ERROR_SUCCESS)
			{
				RegCloseKey(OpenedKey);
				return FALSE;
			}

			if (RegQueryValueEx(FinalOpenedKey, FinalKeyString,0,0, 
				NULL, &dw) == ERROR_SUCCESS)
			{
				
				lpsz = calloc(dw,sizeof(TCHAR));
				if(RegQueryValueEx(FinalOpenedKey, FinalKeyString, 0,0, (LPBYTE)lpsz, &dw) == ERROR_SUCCESS)
				{
					str = lpsz;
					if (strlen(str) > 36)//Dong changed 35 to 36
					{
						char compString35[40], compString33[40],compString36[40];
						strncpy(compString35,str,35);
						compString35[35] = (char)NULL;
						strncpy(compString33,str,33);
						compString33[33] = (char)NULL;
						strncpy(compString36,str,36);
						compString36[36] = (char)NULL;
						if ((strcmp(compString35,_T("PCMCIA\\QRS_DIAGNOSTIC_LLC-SPIROCARD")) == 0)
							||(strcmp(compString36,_T("PCMCIA\\QRS_DIAGNOSTIC_LLC-SPIROXCARD")) == 0)
							||(strcmp(compString33,_T("PCMCIA\\QRS_DIAGNOSTIC_LLC-OXICARD")) == 0))
						{
							RegCloseKey(FinalOpenedKey);
							RegCloseKey(OpenedKey);
							free(lpsz);
							return TRUE;
						}
					}
				}
				free(lpsz);
			}
			if (NumberOfKeys > 0)
				NumberOfKeys--;
			StringSize = 256;
			RegCloseKey(FinalOpenedKey);
		}
	RegCloseKey(OpenedKey);

return FALSE;
}

/*
FUNCTION SystemNT-

  This function checks to see if the OS is NT based.  This includes Windows NT, Windows 2000,
  and Windows XP.  Certain items in the Open_Driver function are only for Win9x systems,
  so this function prevents those items from attempting to execute under NT based systems.



*/
BOOLEAN SystemNT()
{
	
   OSVERSIONINFOEX osvi;
   BOOL bOsVersionInfoEx;

   // Try calling GetVersionEx using the OSVERSIONINFOEX structure,
   // which is supported on Windows 2000.
   //
   // If that fails, try using the OSVERSIONINFO structure.

   ZeroMemory(&osvi, sizeof(OSVERSIONINFOEX));
   osvi.dwOSVersionInfoSize = sizeof(OSVERSIONINFOEX);

   if( !(bOsVersionInfoEx = GetVersionEx ((OSVERSIONINFO *) &osvi)) )
   {
      // If OSVERSIONINFOEX doesn't work, try OSVERSIONINFO.

      osvi.dwOSVersionInfoSize = sizeof (OSVERSIONINFO);
      if (! GetVersionEx ( (OSVERSIONINFO *) &osvi) ) 
         return FALSE;
   }

   if (osvi.dwPlatformId == VER_PLATFORM_WIN32_NT)
	   return TRUE;
	else
		return FALSE;

}


/*   
	OPENING THE DRIVER
	The Windows API function CreateFile opens an installable device driver. 
	It returns a handle that can be used to access the driver. 

	WinCE:
	SpiroHandle = CreateFile( TEXT("PFT1:"), (GENERIC_READ | GENERIC_WRITE), 0,
	NULL, OPEN_EXISTING, 0, NULL);

	Win95/98/ME/2000:
	
	hDevice = CreateFile(_T("\\\\.\\SPIROCRD"),GENERIC_READ|GENERIC_WRITE,
		0,
		NULL,
		OPEN_ALWAYS,
		FILE_ATTRIBUTE_NORMAL,
		NULL);


	Syntax 

	HANDLE CreateFile(
	LPCTSTR lpFileName,
	DWORD dwDesiredAccess,
	DWORD dwShareMode,
	LPSECURITY_ATTRIBUTES lpSecurityAttributes,
	DWORD dwCreationDistribution,
	DWORD dwFlagsAndAttributes,
	HANDLE hTemplateFile 
	); 

	Parameters 

	lpFileName: 

	Pointer to a null-terminated string that specifies the name of the device driver to open.
	WinCE: TEXT("PFT1:")
	Win95: "\\\\.\\SPIROCRD.VXD"

	dwDesiredAccess 

	Type of access to the driver. An application can obtain read access, write access, read/write access, or device query access. You can use the following flag constants to build a value for this parameter.

	0 - Allows an application to query device attributes without actually accessing the device. 

	GENERIC_READ - Specifies read access to the driver. Data can be read from the driver. 

	GENERIC_WRITE - Specifies write access to the driver. Data can be written to the driver. 

	WinCE: Both GENERIC_READ and GENERIC_WRITE must be set to obtain read/write access.
	Win95: This parameter must be set to 0.

	dwShareMode 

	How this driver can be shared.

	0 - Prevents the driver from being shared.

	WinCE: This parameter must be set to 0:
	Win95: This parameter must be set to 0:

	LpSecurityAttributes: 

	This parameter is ignored and should be set to NULL. 

	dwCreationDistribution: 

	CREATE_NEW - Creates a new file.  The function fails if the specified file already exists.

	OPEN_EXISTING - Opens the driver. The function fails if the driver does not exist. 

	WinCE: This parameter should be set to OPEN_EXISTING
	Win95:  This parameter should be set to CREATE_NEW

	dwFlagsAndAttributes: 

	FILE_FLAG_DELETE_ON_CLOSE - flag indicates that the driver is unloaded when CloseHandle is called. 

	WinCE: Driver attributes should be set to zero. 
	Win95: This parameter should be set to FILE_FLAG_DELETE_ON_CLOSE.
 
	HTemplateFile :

	Ignored and should be set to 0. 

	Return Values :

	If the function succeeds, the return value is an open handle to the specified driver. If the function fails, 
	the return value is INVALID_HANDLE_VALUE. To get extended error information, call GetLastError.

	WinCE: If the specified driver is open before the function call and dwCreationDistribution is 
	OPEN_ALWAYS, a call to GetLastError returns ERROR_ALREADY_EXISTS (even though the function has succeeded). 
	If the driver is not open before the call, GetLastError returns zero.
*/
HANDLE Open_Driver()
{
	HANDLE hDevice;
	
	if (!SystemNT())//DON't Do this for WDM driver on 2000/XP- HKEY_DYN_DATA does not exist!
	{
		if (!CardInserted())
		{
			MessageBox(NULL, TEXT("Testapp: Card is not inserted Properly.\r\n"), NULL, MB_OK ); 
			return INVALID_HANDLE_VALUE;
		}
	}
		
	hDevice = CreateFile(_T("\\\\.\\SPIROCRD"),GENERIC_READ|GENERIC_WRITE,
		0,
		NULL,
		OPEN_ALWAYS,
		FILE_ATTRIBUTE_NORMAL,
		NULL);
	if (hDevice == INVALID_HANDLE_VALUE) 
	{
		MessageBox(NULL, TEXT("Testapp: error opening device.\r\n"), NULL, MB_OK ); 
	}
	if (!SystemNT())//311 patch
	{
		MsgWaitForMultipleObjects( 0,    // One event to wait for
			   NULL,        // The array of events
			   FALSE,          // Wait for 1 event
			   400,       // Timeout value
			  0);   // Any message wakes up
	}
	return hDevice;
}



/* 
	DRIVER I/O COMMUNICATION

	All communication operates in a master-slave mode. The host application (the master) is responsible
	for initiating every communication transaction to which the driver (the slave) is expected to respond.
	A communication transaction is initiated by the host sending a I/O command, possibly including one or more
	parameters to the driver. The driver is expected to respond to the command. The response may be one or more
	bytes long.

	The Windows CE API function DeviceIoControl is used to communicate with the installable device driver. 
	The DeviceIoControl function sends a control code directly to a specified device driver, causing the
	corresponding device to perform the specified operation. 

	Syntax 

	BOOL DeviceIoControl(
	HANDLE hDevice,
	DWORD dwIoControlCode,
	LPVOID lpInBuffer,
	DWORD nInBufferSize,
	LPVOID lpOutBuffer,
	DWORD nOutBufferSize,
	LPDWORD lpBytesReturned,
	LPOVERLAPPED lpOverlapped
	); 

	Parameters 

	hDevice: 

	Handle to the device that is to perform the operation. Call the CreateFile function to obtain
	a device handle. 


	dwIoControlCode:

	Control code for the operation. This value identifies the specific operation to be performed
	and the type of device on which the operation is to be performed. 

	For more detailed information on each control code, see I/O Control Commands topic. In particular,
	each topic provides details on the usage of the lpInBuffer, nInBufferSize, lpOutBuffer, nOutBufferSize, 
	and lpBytesReturned parameters. 


	LpInBuffer: - Pointer to a buffer that contains the data required to perform the operation. 
	This parameter can be NULL if the dwIoControlCode parameter specifies an operation that does
	not require input data. 

	NInBufferSize: - Size, in bytes, of the buffer pointed to by lpInBuffer. 

	LpOutBuffer: - Pointer to a buffer that receives the operation's output data. 
	This parameter can be NULL if the dwIoControlCode parameter specifies an operation 
	that does not produce output data. 

	nOutBufferSize: - Size, in bytes, of the buffer pointed to by lpOutBuffer. 

	lpBytesReturned: - Pointer to a variable that receives the size, in bytes, 
	of the data stored into the buffer pointed to by lpOutBuffer. 

	This parameter cannot be NULL. Even when an operation produces no output data, and lpOutBuffer 
	can be NULL, the DeviceIoControl function makes use of the variable pointed to by lpBytesReturned. 
	After such an operation, the value of the variable is without meaning. 


	lpOverlapped 

	Ignored, should be set to NULL. 



	Return Values 

	If the function succeeds, the return value is TRUE. If the function fails, the return value is FALSE. 
	To get extended error information, call GetLastError. 

*/

/*

	RESET_SPIROCARD
	Resets the SpiroCard driver to power-up state.

	LpInBuffer  = NULL 
	NInBufferSize = 0 (Zero)
	LpOutBuffer = NULL 
	nOutBufferSize = 0 (Zero)
*/
int ResetSpirocard(HANDLE hDevice) 
{
	DWORD bytesReturned;
	BYTE InBuffer[50];
	BYTE OutBuffer[12001];
	DWORD nInBufferSize = 10;
	DWORD nOutBufferSize = 500;
	
	//MessageBox(NULL, TEXT("About to RESET_SPIROCARD"), TEXT("STATUS"), MB_OK);
	if (!DeviceIoControl(hDevice, (DWORD)RESET_SPIROCARD, InBuffer, nInBufferSize, OutBuffer, nOutBufferSize,&bytesReturned, NULL))
	{
		MessageBox(NULL, TEXT("Testapp: RESET_SPIROCARD error in device ioctl.\r\n"), NULL, MB_OK ); 
		return 0;
	}
	return 1;
}
/* 
	VERSION
	Report the driver version id (e.g. A1.0).

	LpInBuffer  = NULL 

	NInBufferSize = 0 (Zero)

	LpOutBuffer = Pointer to a short that will contain the version number.

	nOutBufferSize = LpOutBuffer 2 bytes.
	
	NOTE:  The DeviceIOControl returns the version in a short value.  This value
		   must be manipulated into a float to get the actual version number.
		   The first byte is the major version, and the second byte
		   is the minor version.

 */

int DriverVersion(HANDLE hDevice)
{
	DWORD BytesReturned;
	short version = 0;
	short topNum,bottomNum;
	float bottomFloat,versionNumber;
	char MessageBuffer[300];

	//MessageBox(NULL, TEXT("About to get driver VERSION"), TEXT("STATUS"), MB_OK);
	if (!DeviceIoControl(hDevice,VERSION, NULL, 0, &version,2,&BytesReturned,NULL))
		{
			//Handle Errors
			MessageBox(NULL, TEXT("Testapp: VERSION error in device ioctl.\r\n"), NULL, MB_OK ); 
			return 0;
		}
	topNum = version >> 8;
	bottomNum = version & 0x00FF;
	bottomFloat = bottomNum/(float)100.0;
	versionNumber = (float)topNum + bottomFloat;

	sprintf(MessageBuffer,TEXT("The driver version is %.2f"),versionNumber);
	MessageBox(NULL,MessageBuffer,TEXT("STATUS"),MB_OK);
	return 1;
}
/* 
	BTPS_CORRECTION

	Set BTPS Correction based on the current air temperature.

	LpInBuffer = Pointer to a 2 member short array that contains 1 parameter showing 
				 that BTPS is enabled, and another parameter which is the current
				 air temperature in degrees Farenheit.

	LpOutBuffer = Pointer to a 2 byte buffer that indicates whether the call to this
				  function succeeded.


*/
int BTPSCorrection (HANDLE hDevice)
{
	short InBuffer[2];
	short CurrentAirTemperature = 72;
	short OutBuffer = 99;
	DWORD bytesReturned;
	char MessageBuffer[300];

	*((short *)InBuffer+0) = BTPS_ENABLED;
	*((short *)InBuffer+1) = CurrentAirTemperature;

	wsprintf( MessageBuffer, TEXT("About to apply BTPS_CORRECTION, current air temp = %ld"), *((short *)InBuffer+1));
	//MessageBox(NULL, MessageBuffer, TEXT("STATUS"), MB_OK);

	if (!DeviceIoControl(hDevice, (DWORD)BTPS_CORRECTION, InBuffer, 4, &OutBuffer, 2, 
		&bytesReturned, NULL) || OutBuffer == FALSE)
	{
		MessageBox(NULL, TEXT("Testapp: Error in BTPS_CORRECTION.\r\n"), TEXT("STATUS"), MB_OK ); 
		return 0;
	}
	return 1;
}
/* 
	SET_MOUTHPIECE

	Set the mouthpiece correction factor for all subsequent driver operations.

	LpInBuffer  = Pointer to the buffer containing the 4 byte unsigned long value for the mouthpiece correction.  For example, a mouthpiece correction of 3535-3674 is formated as a unsigned long 35353674.
	
	NInBufferSize = 4
	
	LpOutBuffer = NULL

	nOutBufferSize = 0

*/
int SetMouthpiece(HANDLE hDevice,LPSTR pszCmdLine)
{
	short tempBuffer[20];
	long InBuffer;
	short OutBuffer = 99;
	DWORD bytesReturned;
	char MessageBuffer[300];

 	tempBuffer[0] = *(pszCmdLine+0);
	tempBuffer[1] = *(pszCmdLine+1);
	tempBuffer[2] = *(pszCmdLine+2);
	tempBuffer[3] = *(pszCmdLine+3);
	tempBuffer[4] = *(pszCmdLine+4);
	tempBuffer[5] = *(pszCmdLine+5);
	tempBuffer[6] = *(pszCmdLine+6);
	tempBuffer[7] = *(pszCmdLine+7); 
	tempBuffer[8] = 0;
//	InBuffer = _tcstol(tempBuffer, &stopstring, 10);
	InBuffer = (long)wcstod(tempBuffer,NULL);
	wsprintf( MessageBuffer, TEXT("About to call driver. Mouthpiece number = %ld"), InBuffer);
	//MessageBox(NULL, MessageBuffer, TEXT("STATUS"), MB_OK ); 
	//MessageBox(NULL, TEXT("About to SET_MOUTHPIECE"), TEXT("STATUS"), MB_OK);
	if (InBuffer == 0)
	{
		MessageBox(NULL, TEXT("Mouthpiece not set.  Put Mouthpiece Number in command line to set it properly.  Function ignored."), TEXT("STATUS"), MB_OK);
		return 1;
	}
	
	
	if (!DeviceIoControl(hDevice, (DWORD)SET_MOUTHPIECE, &InBuffer, 4, &OutBuffer, 2, 
		&bytesReturned, NULL) || OutBuffer == FALSE)
	{
		MessageBox(NULL, TEXT("Testapp: Error in SET_MOUTHPIECE.\r\n"), NULL, MB_OK ); 
		return 0;
	}
	return 1;
}

/* 

	START_ZERO

	Perform unit zeroing (to be executed prior to every maneuver). The driver sets the state of an internal ZEROED flag upon completion or timeout of the zeroing sequence (which is accessed through the I/O command ZERO_DONE).

	Note: The driver resets the state of  the ZEROED flag  to ZERO_NOT_STARTED when the when the following I/O commands have been issued: RESET_SPIROCARD, SET_MOUTHPIECE,  START_FVC, START_MVV, START_SVC.  The correction factor must be set to a correct value prior to the invocation of this command.

	LpInBuffer  = NULL 

	NInBufferSize = 0 (Zero)

	LpOutBuffer = NULL 

	nOutBufferSize = 0 (Zero)
*/

int StartZero(HANDLE hDevice)
{
	DWORD bytesReturned;
	
	//MessageBox(NULL, TEXT("About to START_ZERO"), TEXT("STATUS"), MB_OK);
	if (!DeviceIoControl(hDevice, (DWORD)START_ZERO, NULL, 0, NULL, 0, 
		&bytesReturned, NULL))
	{
		MessageBox(NULL, TEXT("Testapp: error in device ioctl.\r\n"), NULL, MB_OK ); 
		return 0;
	}
	return 1;
}
/* 

	ZERO_DONE
	Reports the state of the internal ZEROED flag indicating status of the
	zeroing operation.

	LpInBuffer  = NULL 

	NInBufferSize = 0 (Zero)

	LpOutBuffer = Pointer to a 2 byte buffer containing the status of the
	zeroing operation:
	ZERO_DONE
	ZERO_NOT_DONE
	ZERO_NOT_STARTED
	ZERO_TIMEOUT
	nOutBufferSize = 2

	Note: The zeroing operation will time-out after 60 seconds.


*/
int ZeroDone(HANDLE hDevice)
{
	DWORD bytesReturned = 0;
	BYTE OutBuffer[2];
	*OutBuffer = 99;
	

	MessageBox(NULL, TEXT("Testapp: About start calling ZERO_DONE calls.\r\n"), TEXT("STATUS"), MB_OK);
	do 
	{
		if (!DeviceIoControl(hDevice, (DWORD)ZERO_DONE, NULL, 0, OutBuffer, 2, 
			&bytesReturned, NULL))
		{
			MessageBox(NULL, TEXT("Testapp: error in device ioctl.\r\n"), NULL, MB_OK ); 
			return 0;
		}
	} while (*(short *)OutBuffer != ZERO_COMPLETE && *(short *)OutBuffer != ZERO_TIMEOUT);

	if (*(short *)OutBuffer == ZERO_COMPLETE)
		MessageBox(NULL, TEXT("Testapp: ZERO_COMPLETE.\r\n"), TEXT("STATUS"), MB_OK);
	else
		MessageBox(NULL, TEXT("Testapp: ZERO_TIMEOUT.\r\n"), TEXT("STATUS"), MB_OK);
	return 1;

}
/* 
	START_FVC
	Start collecting flow and volume samples for an FVC maneuver 
	(the patient is expected to perform the maneuver). The driver continuously 
	collects flow values from the SpiroCard and calculates  the volume accumulated 
	every 10 msec.  The application must use the FLOW_VOLUME_SAMPLES command to 
	receive any new (samples collected since the last call to the driver with 
	FLOW_VOLUME_SAMPLES command) flow and volume samples collected from the SpiroCard.
	
	Note: The START_ZERO command must be invoked prior to this command
	and the internal state of the ZEROED flag must be set to ZERO_DONE.

	LpInBuffer  = NULL 
	
	NInBufferSize = 0 (Zero)
	
	LpOutBuffer = NULL 
	
	nOutBufferSize = 0 (Zero)
*/

int StartFVC(HANDLE hDevice)
{
	DWORD bytesReturned;
	//MessageBox(NULL, TEXT("Testapp: About to call START_FVC.\r\n"), TEXT("STATUS"), MB_OK);
	if (!DeviceIoControl(hDevice, (DWORD)START_FVC, NULL, 0, NULL, 0, 
		&bytesReturned, NULL))
	{
		MessageBox(NULL, TEXT("Testapp: START_FVC error in device ioctl.\r\n"), NULL, MB_OK ); 
		return 0;
	}
	return 1;
}

/* 
	START_MVV
	Start collecting flow and volume samples for an MVV maneuver
	(the patient is expected to perform the maneuver). The driver continuously 
	collects flow values from the SpiroCard and calculates  the volume accumulated 
	every 10 msec.  The application must use the FLOW_VOLUME_SAMPLES command to 
	receive any new (samples collected since the last call to the driver with 
	FLOW_VOLUME_SAMPLES command) flow and volume samples collected from the SpiroCard.
	Note: The START_ZERO command must be invoked prior to this command and the 
	internal state of the ZEROED flag must be set to ZERO_DONE.

	LpInBuffer  = NULL 
	
	NInBufferSize = 0 (Zero)
	
	LpOutBuffer = NULL 
	
	nOutBufferSize = 0 (Zero)
*/

int StartMVV(HANDLE hDevice)
{
	DWORD bytesReturned;
	//MessageBox(NULL, TEXT("Testapp: About to call START_MVV.\r\n"), TEXT("STATUS"), MB_OK);
	if (!DeviceIoControl(hDevice, (DWORD)START_MVV, NULL, 0, NULL, 0, 
		&bytesReturned, NULL))
	{
		MessageBox(NULL, TEXT("Testapp: START_MVV error in device ioctl.\r\n"), NULL, MB_OK ); 
		return 0;
	}
	return 1;
}
/* 
	START_SVC
	Start collecting flow and volume samples for an SVC maneuver 
	(the patient is expected to perform the maneuver). The driver 
	continuously collects flow values from the SpiroCard and calculates  t
	he volume accumulated every 10 msec.  The application must use the 
	FLOW_VOLUME_SAMPLES command to receive any new (samples collected since the 
	last call to the driver with FLOW_VOLUME_SAMPLES command) flow and volume 
	samples collected from the SpiroCard.
	
	Note: The START_ZERO command must be invoked prior to this command and the internal state of the ZEROED flag must be set to ZERO_DONE.

	LpInBuffer  = NULL 
	
	NInBufferSize = 0 (Zero)
	
	LpOutBuffer = NULL 
	
	nOutBufferSize = 0 (Zero)

*/
int StartSVC(HANDLE hDevice)
{
	DWORD bytesReturned;
	//MessageBox(NULL, TEXT("Testapp: About to call START_SVC.\r\n"), TEXT("STATUS"), MB_OK);
	if (!DeviceIoControl(hDevice, (DWORD)START_SVC, NULL, 0, NULL, 0, 
		&bytesReturned, NULL))
	{
		MessageBox(NULL, TEXT("Testapp: START_SVC error in device ioctl.\r\n"), NULL, MB_OK ); 
		return 0;
	}
	return 1;
}
/* 
	FLOW_VOLUME_SAMPLES
	The driver continuously collects flow values from the SpiroCard and 
	calculates  the volume accumulated every 10 msec.  The driver buffers 
	the samples and reports all of the samples collected from the SpiroCard 
	since the last call made to the driver using the FLOW_VOLUME_SAMPLES command.  
	The driver will not report the flow or volume until the patient starts to exhale.
	The flow samples are returned in signed 2 byte integer format in 1/1000th of a 
	liter/sec and the volume samples are returned in signed 2 byte integer format 
	in 1/1000th of a liter.  When the maneuver has finished the first 2 byte integer
	in the return buffer is set to TRUE.

	LpInBuffer  = NULL 
	NInBufferSize = 0 (Zero)
	LpOutBuffer = Pointer to a 2 byte integer array with the following format:

	Array Index	Contents
	0		end of test flag
	1		flow value at time = (-n * 0.01)
	2		volume value at time = (-n * 0.01)
	3		flow value at time = ((1-n) * 0.01)
	4		volume value at time = ((1-n) * 0.01)
	.		.
	.		.
	.		.
	2n - 1		flow value at time = 0.0 (current time)
	2n		volume value at time = 0.0 (current time)

	where n = number of samples collected since the last call made to the driver using the FLOW_VOLUME_SAMPLES command.
	nOutBufferSize = 6001


*/
int FlowVolumeSamples(HANDLE hDevice)
{

	BYTE OutBuffer[6001];
	DWORD bytesReturned;
	char MessageBuffer[300];
	//MessageBox(NULL, TEXT("About to call FLOW_VOLUME_SAMPLES.  Perform a test and then press OK to view the results.\r\n"), TEXT("STATUS"), MB_OK);
	do
	{
		if (!DeviceIoControl(hDevice, (DWORD)FLOW_VOLUME_SAMPLES, NULL, 0, OutBuffer, 6001, 
			&bytesReturned, NULL))
		{
			MessageBox(NULL, TEXT("Testapp: FLOW_VOLUME_SAMPLES error in device ioctl.\r\n"), NULL, MB_OK ); 
			return 0;
		}
		wsprintf(MessageBuffer, TEXT("number of Flow Volume samples = %d, 1st sample = %d, 2nd sample = %d, 3rd sample = %d\r\n"), (short)bytesReturned, *((short *)OutBuffer+1), *((short *)OutBuffer+2), *((short *)OutBuffer+3));
		MessageBox(NULL, MessageBuffer, TEXT("STATUS"), MB_OK ); 


	} while (*((short *)OutBuffer) == FALSE);

	return 1;
}
/* 
	FLOW_BUFFER
	Return the flow sample buffer for the previous FVC, MVV, or SVC test performed.
	Samples in the buffer correspond to the airflow through the mouthpiece every 10 
	msec (100 samples/second). The samples are a 2 byte signed integer in 1/1000th 
	of a liter/sec values. Positive numbers correspond to expiratory flow, negative 
	numbers correspond to inspiratory flow. For example: 5250 corresponds to an 
	expiratory flow of  5.250 l/sec and -6120 corresponds to an inspiratory flow 
	of 6.120 l/sec.

	LpInBuffer  = NULL 

	NInBufferSize = 0 (Zero)

	LpOutBuffer = Pointer to a 2 byte integer array containing the flow values every
	10 msec.

	nOutBufferSize = 3000
*/
int FlowBuffer(HANDLE hDevice)
{
	BYTE OutBuffer[3000];
	DWORD bytesReturned;
	char MessageBuffer[300];

	//MessageBox(NULL, TEXT("About to call FLOW_BUFFER.\r\n"), TEXT("STATUS"), MB_OK);

	if (!DeviceIoControl(hDevice, (DWORD)FLOW_BUFFER, NULL, 0, OutBuffer, 3000, 
		&bytesReturned, NULL))
	{
		MessageBox(NULL, TEXT("Testapp: error in device ioctl.\r\n"), NULL, MB_OK );
		return 0;
	}


	wsprintf(MessageBuffer, TEXT("number of Flow Buffer samples = %d, 1st sample = %d, 2nd sample = %d, 3rd sample = %d\r\n"), (short)bytesReturned, *((short *)OutBuffer+1), *((short *)OutBuffer+2), *((short *)OutBuffer+3));
	MessageBox(NULL, MessageBuffer, TEXT("STATUS"), MB_OK ); 

	return 1;

}

/* 
	VOLUME_BUFFER
	Return the accumulated volume sample buffer for the previous FVC, 
	MVV, or SVC test performed. Samples in the buffer correspond to the 
	accumulated volume through the mouthpiece at 10 msec (100 samples/second) 
	intervals. The samples are a 2 byte signed integer in 1/1000th of a liter values. 
	Positive numbers correspond to expiratory volume, negative numbers correspond to 
	inspiratory volume. For example: 5250 corresponds to an expiratory volume of 
	 5.250 liters and -6120 corresponds to an inspiratory volume of 6.120 liter.

	LpInBuffer  = NULL 
	NInBufferSize = 0 (Zero)
	LpOutBuffer = Pointer to a 2 byte integer array containing the accumulated volume 
	values every 10 msec.
	nOutBufferSize = 3000
*/
int VolumeBuffer(HANDLE hDevice)
{
	BYTE OutBuffer[3000];
	DWORD bytesReturned;
	char MessageBuffer[300];

	//MessageBox(NULL, TEXT("About to call VOLUME_BUFFER.\r\n"), TEXT("STATUS"), MB_OK);
	
	if (!DeviceIoControl(hDevice, (DWORD)VOLUME_BUFFER, NULL, 0, OutBuffer, 3000, 
		&bytesReturned, NULL))
	{
		MessageBox(NULL, TEXT("Testapp: error in device ioctl.\r\n"), NULL, MB_OK );
		return 0;
	}
	wsprintf(MessageBuffer, TEXT("number of Volume Buffer samples = %d, 1st sample = %d, 2nd sample = %d, 3rd sample = %d\r\n"), (short)bytesReturned, *((short *)OutBuffer+1), *((short *)OutBuffer+2), *((short *)OutBuffer+3));
	MessageBox(NULL, MessageBuffer, TEXT("STATUS"), MB_OK ); 
	return 1;

}
/*
	FVC_RESULTS
	Return the results of the last FVC maneuver. The driver 
	returns 18 2-byte signed integer numbers representing the results of the most 
	recent maneuver performed. The numbers represent thousandth parts of the actual
	values (Ni=round(Vi*1000)).

	Buffer Index	Result Value
	0		FVC
	1		PEFR
	2		FEV05
	3		FEV1
	4		FEV3
	5		FEF25
	6		FEF50
	7		FEF75
	8		FEF25-75
	9		FIVC
	10		PIFR
	11		EXTRAPOLATED VOLUME
	12		FIV05
	13		FIV1
	14		FIV3
	15		FIF0.2-1.2
	16		FIF50
	17		FIF25-75

	LpInBuffer  = NULL 

	NInBufferSize = 0 (Zero)

	LpOutBuffer = Pointer to a 2 byte signed integer array containing the FVC 
	maneuver results.
	
	nOutBufferSize = 36


 */
int FVCResults(HANDLE hDevice)
{
	short OutBuffer[18];
	char MessageBuffer[300];
	DWORD bytesReturned;
	if (!DeviceIoControl(hDevice, (DWORD)FVC_RESULTS, NULL, 0, OutBuffer, 36, 
		&bytesReturned, NULL))
	{
		MessageBox(NULL, TEXT("Testapp: FVC_RESULTS error in device ioctl.\r\n"), NULL, MB_OK ); 
		return 0;
	}
	
	//Write the results to screen
	wsprintf( MessageBuffer, TEXT("FVC = %d, PEF = %d, FEV05 = %d, FEV1 = %d, FEV3 = %d, FEF25 = %d, FEF50 = %d, FEF75 = %d, FEF25_75 = %d, FIVC = %d, PIF = %d, Extrapolated vol = %d\r\n\r\n"), 
		*(OutBuffer+0), *(OutBuffer+1), *(OutBuffer+2), *(OutBuffer+3), *(OutBuffer+4), *(OutBuffer+5), *(OutBuffer+6), *(OutBuffer+7), *(OutBuffer+8), *(OutBuffer+9), *(OutBuffer+10), *(OutBuffer+11), *(OutBuffer+12), *(OutBuffer+13), *(OutBuffer+14), *(OutBuffer+15), *(OutBuffer+16), *(OutBuffer+17));
	MessageBox(NULL, MessageBuffer, TEXT("STATUS"), MB_OK ); 
	
	
	
	return 1;
	

}
/* 
	MVV_RESULTS
	Return the results of the last MVV maneuver. The driver returns 
	3 4-byte signed integer numbers representing the results of the most
	recent maneuver performed. The numbers represent thousandth parts of 
	the actual values (Ni=round(Vi*1000)).

	Buffer Index	Result Value
	0		MVV
	1		MTV
	2		RR

	LpInBuffer  = NULL 
	NInBufferSize = 0 (Zero)
	LpOutBuffer = Pointer to a 4 byte signed integer array containing the
	MVV maneuver results.

	nOutBufferSize = 12
*/

int MVVResults(HANDLE hDevice)
{
	long OutBuffer[3];
	char MessageBuffer[300];
	DWORD bytesReturned;
	if (!DeviceIoControl(hDevice, (DWORD)MVV_RESULTS, NULL, 0, OutBuffer, 12, 
		&bytesReturned, NULL))
	{
		MessageBox(NULL, TEXT("Testapp: MVV_RESULTS error in device ioctl.\r\n"), NULL, MB_OK ); 
		return 0;
	}

	wsprintf( MessageBuffer, TEXT("MVV = %ld, MTV = %ld, RR = %ld\r\n"), *(OutBuffer+0), *(OutBuffer+1), *(OutBuffer+2));
	MessageBox(NULL, MessageBuffer, TEXT("STATUS"), MB_OK ); 

	return 1;
	

}
/*
	SVC_RESULTS
	Return the results of the last SVC maneuver. The driver returns 
	a 2-byte signed integer number representing the result of the most 
	recent maneuver performed. The number represent thousandth parts of 
	the actual values (Ni=round(Vi*1000)).
	Buffer Index	Result Value
	0		SVC

	LpInBuffer  = NULL 

	NInBufferSize = 0 (Zero)

	LpOutBuffer = Pointer to a 2 byte signed integer array containing the 
	SVC maneuver results.

	nOutBufferSize = 2

*/
int SVCResults(HANDLE hDevice)
{
	short OutBuffer;
	char MessageBuffer[300];
	DWORD bytesReturned;
	if (!DeviceIoControl(hDevice, (DWORD)SVC_RESULTS, NULL, 0, &OutBuffer, 2, 
		&bytesReturned, NULL))
	{
		MessageBox(NULL, TEXT("Testapp: SVC_RESULTS error in device ioctl.\r\n"), NULL, MB_OK ); 
		return 0;
	}

	wsprintf( MessageBuffer, TEXT("SVC = %ld\r\n"), OutBuffer);
	MessageBox(NULL, MessageBuffer, TEXT("STATUS"), MB_OK ); 
	return 1;
	

}
/*
SPIROCARD_SLEEP
Sets the SpiroCard, OxiCard or SpirOxCard into sleep mode.

LpInBuffer  = NULL 
NInBufferSize = 0 (Zero)
LpOutBuffer = NULL 
nOutBufferSize = 0 (Zero)
*/
int SpiroCard_Sleep(HANDLE hDevice)
{
	DWORD bytesReturned;
	// Close the device.
	//MessageBox(NULL, TEXT("About to set the pc card into sleep mode"), TEXT("SPIROCARD_SLEEP"), MB_OK);
	if (!DeviceIoControl(hDevice, (DWORD)SPIROCARD_SLEEP, NULL, 0, NULL, 0, 
		&bytesReturned, NULL))
	{
		MessageBox(NULL, TEXT("Testapp: error in device ioctl.\r\n"), NULL, MB_OK ); 
		return 0;
	}
	return 1;
}
/* 
CLOSING THE DRIVER

	The Windows function CloseHandle closes an open installable device driver. 
	Syntax 

	BOOL CloseHandle(HANDLE hObject); 

	Parameters 

	hObject - Handle to an open device driver. 

	Return Values 
	If the function succeeds, the return value is TRUE. If the function fails, the return value is FALSE. To get extended error information, call GetLastError. 
*/

int Close_Driver(HANDLE hDevice)
{
	// Close the device.
	//MessageBox(NULL, TEXT("Testapp: About to close the driver.\r\n"), TEXT("STATUS"), MB_OK);
	if (!CloseHandle(hDevice)) 
	{
		MessageBox(NULL, TEXT("Testapp: CloseHandle(hDevice) failed.\r\n"), NULL, MB_OK ); 
		return 0;
	}
	return 1;
}
/* 
	WIN95 Users can ignore this section!
	  
	WinCE Registry Settings

	The following registry entries must be stored in the WinCE registry.  This enables the CE operating system to install the spiroCard driver:

	[HKEY_LOCAL_MACHINE\Drivers\PCMCIA\Detect\05]
	SZ:DLL = spirocrd.dll
	SZ:Entry = DetectionFunction

	[HKEY_LOCAL_MACHINE\Drivers\PCMCIA\SpiroCard]
	SZ:DLL = spirocrd.dll
	SZ:Prefix = PFT

	NOTE: If you run the installer for Pocket Medic, it will take care of entering
	these registry entries.
*/

/*
	READ_PCB_REV
Report the PCB revision id (e.g. A . 2).

	LpInBuffer  = NULL 

	NInBufferSize = 0 (Zero)

	LpOutBuffer = Pointer to a short that will contain the version number.

	nOutBufferSize = 2.
	
	NOTE:  
The DeviceIOControl returns the version in a short value. The first portion of PCB revision is a character,
 the second one is a number. The first byte is the major version, and the second byte is the minor version.
 The forumula is Major_Version = OutBuffer >> 8, Minor_Version = OutBuffer && 0x00FF.  Final Version is like:
 versionString.Format(_T("%c.%d"), Major_Version, Minor_Version)
*/
short GetPCBVersion(HANDLE hDevice)
{
DWORD BytesReturned;
short version = 0;
		if (!DeviceIoControl(hDevice,READ_PCB_REV, NULL, 0, &version,2,&BytesReturned,NULL))
		{
			//Handle Errors
			MessageBox(NULL, TEXT("Testapp: READ_PCB_REV failed.\r\n"), NULL, MB_OK ); 
			return 0;

		}
		else
		{
			char versionString[300], NameString[80];
		short topNum, bottomNum;
		topNum = version >> 8;
		bottomNum = version & 0x00FF;
		//attn- first portion of PCB revision is a character.-
		// formatting is different than versions- leave it at %c.%d
		//versionString.Format(_T("SpiroCard PCB Revision: %c.%d"),topNum,bottomNum);
		sprintf(versionString, TEXT("PCB Version =  %c.%d"), topNum, bottomNum);
		sprintf(NameString, TEXT("GetPCBVersion"));

		MessageBox(NULL,versionString, NameString, MB_OK ); 


		}
 	return(version); 
} 
/*
READ_PC_CARD_TYPE
Return a single point of the data. This data can be equal to SpiroCard_Type, OxiCard_Type, 
SpiroxCard_Type which represent different kind of PC Card in the PC Card socket. 
SpiroxCard can support both Oximetry and Spirometry test. OxiCard and SpiroCard 
can only support Oximetry test or Spriometry test.

	LpInBuffer = NULL
	nInBufferSize = 0
	LpOutBuffer = Pointer to a short that will contain the Card_Type data.
	NOutBufferSize = 2
*/
short ReadPCCardType(HANDLE hDevice)
{
	DWORD bytesReturned;
	short CardType=0;
	//MessageBox(NULL, TEXT("About to Read PC Card Type"), TEXT("READ_PC_CARD_TYPE"), MB_OK);
	if (!DeviceIoControl(hDevice, (DWORD)READ_PC_CARD_TYPE, NULL, 0, &CardType, 2, 
		&bytesReturned, NULL))
	{
		MessageBox(NULL, TEXT("Testapp: error in device ioctl.\r\n"), NULL, MB_OK ); 
		return 0;
	}
	if(CardType==SpiroxCard_Type)
		;//MessageBox(NULL, TEXT("It is SpirOxCard"), TEXT("READ_PC_CARD_TYPE"), MB_OK);
	else if(CardType==OxiCard_Type)
		;//MessageBox(NULL, TEXT("It is OxiCard"), TEXT("READ_PC_CARD_TYPE"), MB_OK);
	else if(CardType==SpiroCard_Type)
		;//MessageBox(NULL, TEXT("It is SpiroCard"), TEXT("READ_PC_CARD_TYPE"), MB_OK);
	else MessageBox(NULL, TEXT("It is an unknow Card "), TEXT("READ_PC_CARD_TYPE"), MB_OK);
	return CardType;
}
/*
OXI_POWER_ON
Turn on the power of Oximetry portion on the OxiCard or SpirOxCard.

LpInBuffer  = NULL 
NInBufferSize = 0 (Zero)
LpOutBuffer = NULL 
nOutBufferSize = 0 (Zero)
*/
short OxiPowerOn(HANDLE hDevice)
{
	DWORD bytesReturned;
	//MessageBox(NULL, TEXT("About to Turn on the power of Oxi module"), TEXT("OXI_POWER_ON"), MB_OK);
	if (!DeviceIoControl(hDevice, (DWORD)OXI_POWER_ON, NULL, 0, NULL, 0, 
		&bytesReturned, NULL))
	{
		MessageBox(NULL, TEXT("Testapp: error in device ioctl.\r\n"), NULL, MB_OK ); 
		return 0;
	}
	return 1;
}
/*
RESET_OXICARD
Resets the OxiCard or SpirOxCard driver to power-up state.

LpInBuffer  = NULL 
NInBufferSize = 0 (Zero)
LpOutBuffer = NULL 
nOutBufferSize = 0 (Zero)
*/
short ResetOxiCard(HANDLE hDevice)
{
	DWORD bytesReturned;
	//MessageBox(NULL, TEXT("About to Reset the Oxi module"), TEXT("RESET_OXICARD"), MB_OK);
	if (!DeviceIoControl(hDevice, (DWORD)RESET_OXICARD, NULL, 0, NULL, 0, 
		&bytesReturned, NULL))
	{
		MessageBox(NULL, TEXT("Testapp: error in device ioctl.\r\n"), NULL, MB_OK ); 
		return 0;
	}
	return 1;
}
/*
PLETH_BUFFER
The driver continuously collects plethysmographic pulse values from the OxiCard or SpirOxCard at the speed of 75 times a second.  The driver buffers the samples and reports all of the samples collected from the Card since the last call made to the driver using the PLETH_BUFFER command.

LpInBuffer  = NULL 
NInBufferSize = 0 (Zero)
LpOutBuffer = Pointer to a  byte unsigned char array containing the values of plethysmography

nOutBufferSize = 75*60*2=9000	(2 Minutes) 
*/
short PlethBuffer(HANDLE hDevice)
{
	DWORD bytesReturned;
	UCHAR itsPlethBuffer[9000];
	char MessageBuffer[300];
	//MessageBox(NULL, TEXT("About to call About to call PLETH_BUFFER"), TEXT("PLETH_BUFFER"), MB_OK);
	if (!DeviceIoControl(hDevice, (DWORD)PLETH_BUFFER, NULL, 0, itsPlethBuffer, 9000,
		&bytesReturned, NULL))
	{
		MessageBox(NULL, TEXT("Testapp: error in device ioctl.\r\n"), NULL, MB_OK ); 
		return 0;
	}
	wsprintf(MessageBuffer, TEXT("number of PlethBuffer samples = %d, 1st sample = %d, 2nd sample = %d, 3rd sample = %d\r\n"), (short)bytesReturned, *((UCHAR *)itsPlethBuffer+1), *((UCHAR *)itsPlethBuffer+2), *((UCHAR *)itsPlethBuffer+3));
	MessageBox(NULL, MessageBuffer, TEXT("PLETH_BUFFER"), MB_OK ); 
	return 1;
}

short OxiPleth(HANDLE hDevice)
{
	DWORD bytesReturned;
	UCHAR itsPlethBuffer[9000];
//	char MessageBuffer[300];
	//MessageBox(NULL, TEXT("About to call About to call PLETH_BUFFER"), TEXT("PLETH_BUFFER"), MB_OK);
	if (!DeviceIoControl(hDevice, (DWORD)PLETH_BUFFER, NULL, 0, itsPlethBuffer, 9000,
		&bytesReturned, NULL))
	{
//		MessageBox(NULL, TEXT("Testapp: error in device ioctl.\r\n"), NULL, MB_OK ); 
		return 0;
	}
//	wsprintf(MessageBuffer, TEXT("number of PlethBuffer samples = %d, 1st sample = %d, 2nd sample = %d, 3rd sample = %d\r\n"), (short)bytesReturned, *((UCHAR *)itsPlethBuffer+1), *((UCHAR *)itsPlethBuffer+2), *((UCHAR *)itsPlethBuffer+3));
//	MessageBox(NULL, MessageBuffer, TEXT("PLETH_BUFFER"), MB_OK ); 
	return (*((UCHAR *)itsPlethBuffer+1));
}

/*
OXI_SPO2

	Return a single point of the current Oxi data.
	LpInBuffer = NULL
	nInBufferSize = 0
	LpOutBuffer = Pointer to a short that will contain the OXI data.
	NOutBufferSize = 2
*/
short OxiSpO2(HANDLE hDevice)
{
	DWORD bytesReturned;
	short spo2Value =0;
	short pulseValue = 0;
	DWORD itsStatus =0;
	int returnValue = 0;
	int i = 0;
	//char MessageBuffer[300];
	//MessageBox(NULL, TEXT("About to call About to call OXI_SP02"), TEXT("OXI_PULSE"), MB_OK);
for ( i = 0; i < 10; i++ ) 
{
		if (!DeviceIoControl(hDevice, (DWORD)OXI_SPO2, NULL, 0, &spo2Value, 2, 
                      		&bytesReturned, NULL))
		{
			MessageBox(NULL, TEXT("Testapp: error in device ioctl OXI_SP02.\r\n"), NULL, MB_OK ); 
			return 0;
		}
		if (!DeviceIoControl(hDevice, (DWORD)OXI_PULSE, NULL, 0, &pulseValue, 2, 
                      		&bytesReturned, NULL))
		{
			MessageBox(NULL, TEXT("Testapp: error in device ioctl OXI_PULSE.\r\n"), NULL, MB_OK ); 
			return 0;
		}
		if (!DeviceIoControl(hDevice, (DWORD)OXI_STATUS, NULL, 0, &itsStatus, 2, 
                      		&bytesReturned, NULL))
		{
			MessageBox(NULL, TEXT("Testapp: error in device ioctl.OXI_STATUS\r\n"), NULL, MB_OK ); 
			return 0;
		}
	//attn: get rid of this eventually.
	// the driver returns 127 to indicate a bad data reading- eventually this will change.
		if ((spo2Value <= 0)||(spo2Value >= 100)||(pulseValue >= MAX_HEART_RATE)||(pulseValue <= 0)||(itsStatus & 0x38))
		{
			spo2Value = -1;
			pulseValue = -1;
			//returnValue = MessageBox(NULL,TEXT("Waiting for valid sp02 data.  Continue?"),TEXT("OXI_SPO2PULSEBUFFER"),MB_YESNO);
			//if (returnValue == IDNO)
				//return 0;
		} 
		else
		{
				//wsprintf(MessageBuffer, TEXT("SpO2Value = %d%%\r\n"), spo2Value);
				//MessageBox(NULL, MessageBuffer, TEXT("OXI_SPO2"), MB_OK );
				return spo2Value;
		}
			
}
	return 0;
}
/*
OXI_PULSE
Return a single point of  the current Pulse data.
LpInBuffer  = NULL.
NInBufferSize = 0
LpOutBuffer = Pointer to a short that will contain the Pulse data.
nOutBufferSize = 2
*/
short OxiPulse(HANDLE hDevice)
{
	DWORD bytesReturned;
	short pulseValue;
	short spo2Value = 0;
	DWORD itsStatus = 0;
	int returnValue = 0;
	int i = 0;
//	char MessageBuffer[300];
//MessageBox(NULL, TEXT("About to call About to call OXI_PULSE"), TEXT("OXI_PULSE"), MB_OK);
for ( i = 0; i < 10; i++ ) 
{
		if (!DeviceIoControl(hDevice, (DWORD)OXI_SPO2, NULL, 0, &spo2Value, 2, 
                      		&bytesReturned, NULL))
		{
			MessageBox(NULL, TEXT("Testapp: error in device ioctl OXI_SPO2.\r\n"), NULL, MB_OK ); 
			return 0;
		}
		if (!DeviceIoControl(hDevice, (DWORD)OXI_PULSE, NULL, 0, &pulseValue, 2, 
                      		&bytesReturned, NULL))
		{
			MessageBox(NULL, TEXT("Testapp: error in device ioctl OXI_PUSE.\r\n"), NULL, MB_OK ); 
			return 0;
		}
		if (!DeviceIoControl(hDevice, (DWORD)OXI_STATUS, NULL, 0, &itsStatus, 2, 
                      		&bytesReturned, NULL))
		{
			MessageBox(NULL, TEXT("Testapp: error in device ioctl OXI_STATUS.\r\n"), NULL, MB_OK ); 
			return 0;
		}
	//attn: get rid of this eventually.
	// the driver returns 127 to indicate a bad data reading- eventually this will change.
		if ((spo2Value <= 0)||(spo2Value >= 100)||(pulseValue >= MAX_HEART_RATE)||(pulseValue <= 0)||(itsStatus & 0x38))
		{
			spo2Value = -1;
			pulseValue = -1;
			//returnValue = MessageBox(NULL,TEXT("Waiting for valid sp02 data.  Continue?"),TEXT("OXI_SPO2PULSEBUFFER"),MB_YESNO);
			//if (returnValue == IDNO)
				//return 0;
		} 
		else
		{		//wsprintf(MessageBuffer, TEXT("PulseValue = %dbpm\r\n"), pulseValue);
				//MessageBox(NULL, MessageBuffer, TEXT("OXI_PULSE"), MB_OK ); 
				return pulseValue;
		}
			
}























	
	return 0;
}
/*
OXI_STATUS
Return a single byte of status of the Oxi module
LpInBuffer  = NULL.
NInBufferSize = 0
LpOutBuffer = Pointer to a unsigned char that will contain the status data. Each bit of the data has the following meaning:
	Bit0: frame Sync, set for 1 of 25,  clear for 2-25 of 25
	Bit1: green perfusion, set if true only during pulse
	Bit2: red perfusion, set if true only during pulse
	Bit3: Sensor alarm, set if true
	Bit4: out of track, set if true
	Bit5: bad pulse, set if true
	Bit6: reserved for 8600 alarms
	Bit7: always set
		Note: bits 1&2 are set for yellow perfusion
nOutBufferSize = 1
*/
short OxiStatus(HANDLE hDevice)
{
	DWORD bytesReturned;
	short itsStatus;
	char MessageBuffer[300];
	//MessageBox(NULL, TEXT("About to call About to call OXI_STATUS"), TEXT("OXI_STATUS"), MB_OK);
	if (!DeviceIoControl(hDevice, (DWORD)OXI_STATUS, NULL, 0, &itsStatus, 2,  
		&bytesReturned, NULL))
	{
		MessageBox(NULL, TEXT("Testapp: error in device ioctl.\r\n"), NULL, MB_OK ); 
		return 0;
	}
	wsprintf(MessageBuffer, TEXT("StatusValue = %d\r\n"), itsStatus);
	MessageBox(NULL, MessageBuffer, TEXT("OXI_STATUS"), MB_OK ); 
	return 1;
}
/*
OXI_SPO2PULSEBUFFER
The driver continuously collects SPO2 and Pulse values from the OxiCard or SpirOxCard 
at the speed of 3 times a second.  The driver buffers the values and reports all of 
the data collected from the Card since the last call made to the driver using the 
OXI_SPO2PULSEBUFFER command.

LpInBuffer  = NULL 
NInBufferSize = 0 (Zero)
LpOutBuffer = Pointer to a 2 byte integer array with the following format:

Array Index	Contents
0		pulse value at time = -n
1		spo2 value at time = -n
2		pulse value at time = 1-n
3		spo2 value at time = 1-n
4		pulse value at time = 2-n
.		.
.		.
.		.
2n - 2		pulse value at time = 0.0 (current time)
2n - 1		spo2 value at time = 0.0 (current time)

where n = number of data collected since the last call made to the driver using the 
OXI_SPO2PULSEBUFFER command.
nOutBufferSize = 3*2*2*60*5=3600 (5 Minutes) 
*/
short OxiSpO2PulseBuffer(HANDLE hDevice)
{
	DWORD bytesReturned = 0;
	short spo2Value = 0;
	short pulseValue = 0;
	short itsStatus = 0;
	int returnValue = 0;
	short  spo2PulseBuffer[Spo2PulseBuffer_Size];//Dong changed 80 to Spo2PulseBuffer_Size  8/23/00
	char MessageBuffer[300];
//MessageBox(NULL, TEXT("About to call About to call OXI_SPO2PULSEBUFFER"), TEXT("OXI_PULSE"), MB_OK);
while (1)
{
		if (!DeviceIoControl(hDevice, (DWORD)OXI_SPO2, NULL, 0, &spo2Value, 2, 
                      		&bytesReturned, NULL))
		{
			MessageBox(NULL, TEXT("Testapp: error in device ioctl.\r\n"), NULL, MB_OK ); 
			return 0;
		}
		if (!DeviceIoControl(hDevice, (DWORD)OXI_PULSE, NULL, 0, &pulseValue, 2, 
                      		&bytesReturned, NULL))
		{
			MessageBox(NULL, TEXT("Testapp: error in device ioctl.\r\n"), NULL, MB_OK ); 
			return 0;
		}
		if (!DeviceIoControl(hDevice, (DWORD)OXI_STATUS, NULL, 0, &itsStatus, 2, 
                      		&bytesReturned, NULL))
		{
			MessageBox(NULL, TEXT("Testapp: error in device ioctl.\r\n"), NULL, MB_OK ); 
			return 0;
		}
	//attn: get rid of this eventually.
	// the driver returns 127 to indicate a bad data reading- eventually this will change.
		if ((spo2Value <= 0)||(spo2Value >= 100)||(pulseValue >= MAX_HEART_RATE)||(pulseValue <= 0)||(itsStatus & 0x38))
		{
			spo2Value = -1;
			pulseValue = -1;
			returnValue = MessageBox(NULL,TEXT("Waiting for valid oxi pulse buffer data. This generally takes 10-15 seconds. Continue?"),TEXT("OXI_SPO2PULSEBUFFER"),MB_YESNO);
			if (returnValue == IDNO)
				return 0;
		} 
		else
			break;
}




	//MessageBox(NULL, TEXT("About to call About to call OXI_SPO2PULSEBUFFER"), TEXT("OXI_SPO2PULSEBUFFER"), MB_OK);
	if (!DeviceIoControl(hDevice, (DWORD)OXI_SPO2PULSEBUFFER, NULL, 0, spo2PulseBuffer, Spo2PulseBuffer_Size*2, 
		&bytesReturned, NULL))
	{
		MessageBox(NULL, TEXT("Testapp: error in device ioctl.\r\n"), NULL, MB_OK ); 
		return 0;
	}
	wsprintf(MessageBuffer, TEXT("number of OXI_SPO2PULSEBUFFER samples = %d, Spo2Value = %d%%, PulseValue = %dbpm...\r\n"), bytesReturned,spo2PulseBuffer[1],spo2PulseBuffer[0]);
	MessageBox(NULL, MessageBuffer, TEXT("OXI_SPO2PULSEBUFFER"), MB_OK ); 
	return 1;
}

HANDLE hDevice = INVALID_HANDLE_VALUE;
int started = 0;

int StartQRS()
{
	short CardType;

	// Open the device.
	hDevice = Open_Driver();		
	if (hDevice == INVALID_HANDLE_VALUE) 
		return 0;
	CardType=0;
	CardType=ReadPCCardType(hDevice);

	if((CardType==SpiroxCard_Type)||(CardType==OxiCard_Type))
	{
		if (!OxiPowerOn(hDevice))
			return 0;
		if (!ResetOxiCard(hDevice))
			return 0;
				
	}
	started = 1;
	return 0;
}

int ReadQRS( int* o2, int* pulse, int* signal ) {
	int result = 0;
	short o2val = OxiSpO2(hDevice);
	short pulseVal = OxiPulse(hDevice);
	short signalVal = OxiPleth(hDevice);
	if (( o2val == 0 ) || ( pulseVal == 0 )) {
		return 0;
	} else {
		*o2 = o2val;
		*pulse = pulseVal;
		*signal = signalVal;
		return 1;
	}
}
			

int StopQRS() {
	if ( !started ) {
		return 0;
	}
	if ( hDevice != INVALID_HANDLE_VALUE ) {
		if (!SpiroCard_Sleep(hDevice))
			return 0;
		if (!Close_Driver(hDevice))
			return 0;
	}

	return 0;
}
